<?php

namespace common\services\Producer;

use common\helpers\Password;
use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\logic\ServiceInterface;
use common\models\Producer;
use common\logic\UserProducer\UserProducerModel;
use common\repositories\ProducerRepository;

class ProducerBuilder extends BaseService implements BuilderInterface
{
    protected ProducerRepository $producerRepository;

    public function __construct()
    {
        $this->producerRepository = $this->loadService(ProducerRepository::class);
    }

    public function init(Producer $producer): void
    {
        $this->initSlug($producer);
        $this->initCode($producer);
    }

    public function initSlug(Producer $producer): void
    {
        $cptSlug = 0 ;
        do {
            $slug = \common\helpers\Url::slugify($producer->name) ;
            if($cptSlug) {
                $slug .= $cptSlug ;
            }
            $producer->slug = $slug ;
            $cptSlug ++ ;
        } while($this->producerRepository->getOneBySlug($producer->slug)) ;
    }

    public function initCode(Producer $producer): void
    {
        $producer->code = Password::generate();
    }

    /**
     * Lie un utilisateur à un producteur.
     *
     * @param integer $id_user
     * @param integer $id_producer
     * @return UserProducerModel
     */
    public static function addUser(int $idUser, int $idProducer, int $bookmark = 1)
    {
        $userProducer = UserProducerModel::searchOne([
            'user_producer.id_user' => $idUser,
            'user_producer.id_producer' => $idProducer
        ]);

        if (!$userProducer) {
            $newUserProducer = new UserProducerModel;
            $newUserProducer->id_producer = $idProducer;
            $newUserProducer->id_user = $idUser;
            $newUserProducer->credit = 0;
            $newUserProducer->active = 1;
            $newUserProducer->bookmark = (int)$bookmark;
            $newUserProducer->save();
        } else {
            if (!$userProducer->active) {
                $userProducer->active = 1;
                $userProducer->save();
            }
        }

        return $userProducer;
    }
}